
init python:
    
    import shutil
    import os.path

    ####################################################################################################################################
    #
    # READ
    #
    ####################################################################################################################################

    def readTransforms():

        if not "transformacion" in globals()["xmlDocs"]: 
            return

        transformsFilename = globals()["xmlDocs"]["transformacion"]
        tree = parseXMLFile( transformsFilename )
        if tree == None: 
            REPORTERROR("Falló la lectura del documento " + transformsFilename)
            return

        for child in tree.getroot():

            t = Transform( child )

            if not t.id in globals()["symt"]:
                globals()["symt"].append(t.id)
            else:
                REPORTERROR("Transformación con ID duplicado: " + t.id)
                return

            transforms[t.id] = t

    def readAnimations():

        if not "animacion" in globals()["xmlDocs"]: 
            return

        animationsFilename = globals()["xmlDocs"]["animacion"]

        tree = parseXMLFile( animationsFilename )
        if tree == None: 
            REPORTERROR("Falló la lectura del documento " + animationsFilename)
            return

        for child in tree.getroot():
            
            a = Animation( child )
            
            if not a.id in globals()["symt"]:
                globals()["symt"].append(a.id)
            else:
                REPORTERROR("Animación con ID duplicado: " + a.id)
                return

            animations[ a.id ] = a

    def readWordDefinitions():

        if not "definicion" in globals()["xmlDocs"]: 
            return

        wordDefinitionsFilename = globals()["xmlDocs"]["definicion"]

        tree = parseXMLFile( wordDefinitionsFilename )
        if tree == None: 
            REPORTERROR("Falló la lectura del documento " + wordDefinitionsFilename)
            return

        for child in tree.getroot():

            w = StoryScene( child )

            if not w.id in globals()["symt"]:
                globals()["symt"].append(w.id)
            else:
                REPORTERROR("Definición con ID duplicado: " + w.id)
                return

            wordDefinitions[w.id] = w

    def readStoryScenes():

        if not "narracion" in globals()["xmlDocs"]: 
            return

        storySceneFilename = globals()["xmlDocs"]["narracion"]

        tree = parseXMLFile( storySceneFilename )
        if tree == None: 
            REPORTERROR("Falló la lectura del documento " + storySceneFilename)
            return

        for child in tree.getroot():
            s = StoryScene( child )

            if not s.id in globals()["symt"]:
                globals()["symt"].append(s.id)
            else:
                REPORTERROR("Escena de narración con ID duplicado: " + s.id)
                return

            storyScenes[s.id] = s

    def readActivityScenes():

        if not "actividad" in globals()["xmlDocs"]: 
            return

        activitySceneFilename = globals()["xmlDocs"]["actividad"]

        tree = parseXMLFile( activitySceneFilename )
        if tree == None: 
            REPORTERROR("Falló la lectura del documento " + activitySceneFilename)
            return

        for child in tree.getroot():
            s = ActivityScene( child )

            if not s.id in globals()["symt"]:
                globals()["symt"].append(s.id)
            else:
                REPORTERROR("Escena de actividad con ID duplicado: " + s.id)
                return

            activityScenes[s.id] = s
    
    def readUIScenes():

        if not "iu" in globals()["xmlDocs"]: 
            return

        UISceneFilename = globals()["xmlDocs"]["iu"]

        tree = parseXMLFile( UISceneFilename )
        if tree == None: 
            REPORTERROR("Falló la lectura del documento " + UISceneFilename)
            return

        for child in tree.getroot():
            s = UIScene( child )

            if not s.id in globals()["symt"]:
                globals()["symt"].append(s.id)            
            else:
                REPORTERROR("Escena de IU con ID duplicado: " + s.id)
                return

            if s.id == "menuprincipal":
                globals()["definedMainMenu"] = True

            UIScenes[s.id] = s

    def readTransitionScenes():

        if not "transicion" in globals()["xmlDocs"]: 
            return

        transitionSceneFilename = globals()["xmlDocs"]["transicion"]

        tree = parseXMLFile( transitionSceneFilename )
        if tree == None: 
            REPORTERROR("Falló la lectura del documento " + transitionSceneFilename)
            return

        for child in tree.getroot():
            s = StoryScene( child )

            if not s.id in globals()["symt"]:
                globals()["symt"].append(s.id)
            else:
                REPORTERROR("Escena de Transición con ID duplicado: " + s.id)
                return

            transitionScenes[s.id] = s

    def readDocuments():
        
        LOG("Leyendo transformaciones...")
        readTransforms()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        LOG("Leyendo animaciones...")
        readAnimations()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        LOG("Leyendo definiciones...")
        readWordDefinitions()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        LOG("Leyendo escenas de narración...")
        readStoryScenes()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        LOG("Leyendo escenas de actividad...")
        readActivityScenes()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        LOG("Leyendo escenas de IU...")
        readUIScenes()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        LOG("Leyendo escenas de transición...")
        readTransitionScenes()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

    ####################################################################################################################################
    #
    # WRITE
    #
    ####################################################################################################################################

    globals()['gccount'] = {
        'Transformaciones':0,
        'Animaciones':0,
        'Definiciones':0,
        'Recursos':0,
        'Escenas de script':0,
        'Escenas de transición':0,
        'Escenas de IU':0
        }

    def setupOutputDirectory():
        LOG("Preparando directorio de salida...")

        if globals()["projectDirectory"] == "":
            REPORTERROR( "No existe un directorio principal" )
            return

        pd = os.path.abspath( os.path.join( globals()["projectDirectory"], os.path.pardir ) )
        pd = os.path.abspath( os.path.join( pd, os.path.pardir ) )
        pd = os.path.abspath( os.path.join( pd, os.path.pardir ) )
        od = globals()["projectOutDirectory"] = os.path.join( pd, globals()["projectName"] )

        LOG("\n     Se intentará generar los archivos del proyecto en: " + od)

        if directoryExists(od):

            LOG("\n     Verificando permiso de escritura en directorio " + od)
            if not directoryIsWrittable( od ):
                REPORTERROR( "El directorio no tiene permiso de escritura" )
                return

            LOG("\n     Eliminando archivos del directorio " + od)
            result = clearDirectory(od)
            if not result:
                REPORTERROR( "Imposible eliminar archivos antiguos en directorio de salida" )
                return
        
        LOG("\n     Creando el directorio " + od)
        result = createRenpyProjectTree( od )
        if not result:
            REPORTERROR( "No pudo crearse el directorio" )
            return

    def generateCodeUtilities():
        LOG("Creando rutinas auxiliares")
        if not generateRenpyUtilityFiles( globals()["projectOutDirectory"] + "/game" ):
            REPORTERROR( "Falló la creación de utilidades" )
            return

    def generateTransforms():

        LOG("Generando transformaciones...")
        
        outTransformsFile = os.path.abspath( os.path.join( globals()["projectOutDirectory"] + "/game/transforms.rpy" ) )

        try:
            with open(outTransformsFile, "w") as f:
                if len(transforms) > 0:
                    f.write("init:\n")
                for tr in transforms:
                    lines = transforms[tr].generate()
                    f.write( "".join(lines))
                    globals()['gccount']['Transformaciones'] += 1
        except:
            REPORTERROR("Falló generación de código en " + outTransformsFile)            

    def generateAnimations():

        LOG("Generando animaciones...")
        
        outAnimationsFile = os.path.abspath( os.path.join( globals()["projectOutDirectory"] + "/game/animations.rpy" ) )

        try:
            with open(outAnimationsFile, "w") as f:
                if len(animations) > 0:
                    f.write("init:\n")
                for animation in animations:
                    lines = animations[animation].generate()
                    f.write( "".join(lines))
                    globals()['gccount']['Animaciones'] += 1
        except:
            REPORTERROR("Falló generación de código en " + outAnimationsFile)            

    def generateWordDefinitions():
        LOG("Generando definiciones de palabras...")
        try:
            outFile = os.path.abspath( os.path.join( globals()["projectOutDirectory"] + "/game/definitions.rpy" ) )

            with open(outFile, "w") as f:
                for definition in globals()["wordDefinitions"]:
                    lines = globals()["wordDefinitions"][definition].generate()
                    f.write( 'label ' + globals()["wordDefinitions"][definition].id + ':\n')
                    f.write(''.join(lines[2:]).encode('UTF-8'))
                    f.write( '    return\n' )
                    globals()['gccount']['Definiciones'] += 1
        except:
            REPORTERROR("Falló generación de código en " + outFile)

    def generateUIScenes():
        LOG("Generando escenas de IU...")

        # escenas especiales
        specialScenes = [ "menuprincipal", "acerca", "opciones", "cargar", "salvar", "infoPuntos", "infoLogros", "infoMenu" ]
        for sceneId in specialScenes:
            if sceneId in globals()["UIScenes"]:
                scene = globals()["UIScenes"][sceneId]
                lines = scene.generate()
                specialFile = os.path.abspath( os.path.join( globals()["projectOutDirectory"] + "/game/scene_" + scene.id + ".rpy" ) )

                try:
                    with open( specialFile, "w" ) as sf:
                        sf.write(''.join(lines).encode('UTF-8'))
                        globals()['gccount']['Escenas de IU'] += 1
                except:
                    REPORTERROR('Falló generación de escena "' + sceneId + '"')

    def getNextScene(scene):
        if scene == None:
            return None

        if scene.next in ["", None]:
            return None

        if not scene.next in globals()["storyScenes"]:
            if not scene.next in globals()["activityScenes"]:
                return None
            else:
                return globals()["activityScenes"][scene.next]
        else:
            return globals()["storyScenes"][scene.next]

        return None

    def generateScriptScenes():
        LOG("Generando escenas de narración y actividad...")

        outFile = os.path.abspath( os.path.join( globals()["projectOutDirectory"] + "/game/script.rpy" ) )
        
        try:
            open( outFile, "w" )
        except:
            REPORTERROR("Imposible escribir en " + outFile)
            return

        # for avoiding referencing loops
        scenesCreated = []
        
        with open( outFile, "a" ) as f:
            try:
                if not globals()["definedMainMenu"]:
                    f.write("\nlabel main_menu:\n")
                    f.write("    return\n\n")

                if 'presentacion' in globals()["storyScenes"]:
                    f.write("\nlabel splashscreen:\n")
                    ps = globals()['storyScenes']['presentacion']
                    lines = ps.generate()
                    f.write(''.join(lines).encode('UTF-8'))
                    f.write("    return\n\n")
                    scenesCreated.append( ps.id )
                    globals()['gccount']['Escenas de script'] += 1

                f.write("label start:\n")
            except Exception, e:
                REPORTERROR("Imposible escribir en " + outFile)
                #raise e
                return


            scene = None

            if 'inicio' in globals()["storyScenes"]:
                scene = globals()["storyScenes"]['inicio']


            while scene != None and (not scene.id in scenesCreated):

                f.write( ("\n    # escena: " + scene.id + "\n").encode('UTF-8') )
                lines = scene.generate()
                f.write(''.join(lines).encode('UTF-8'))
                scenesCreated.append( scene.id )
                globals()['gccount']['Escenas de script'] += 1

                if scene.tran in globals()["transitionScenes"]:
                    f.write('    call ' + scene.tran + '\n')

                scene = getNextScene( scene )

            f.write("\n    $ renpy.quit()\n")
    

    def generateTransitionScenes():
        LOG("Generando escenas de transición...")
        try:
            outFile = os.path.abspath( os.path.join( globals()["projectOutDirectory"] + "/game/transicion.rpy" ) )

            with open(outFile, "w") as f:
                for scene in globals()["transitionScenes"]:
                    lines = globals()["transitionScenes"][scene].generate()
                    f.write( 'label ' + globals()["transitionScenes"][scene].id + ':\n')
                    f.write(''.join(lines).encode('UTF-8'))
                    f.write( '    return\n' )
                    globals()['gccount']['Escenas de transición'] += 1
        except:
            REPORTERROR("Falló la generación de código en " + outFile)

    def generateConfiguration():
        LOG("Ajustando configuración...")
        outFile = os.path.abspath( os.path.join( globals()["projectOutDirectory"] + "/game/options.rpy" ) )
        try:
            with open( outFile, "a") as f:
                lines = []
                lines.append( 'python early:\n')
                lines.append( '    config.save_directory = "' + globals()["projectName"] + '-12012012"\n' )
                lines.append( 'init -1 python hide:\n')
                lines.append( '    config.screen_width = 800\n')
                lines.append( '    config.screen_height = 600\n')
                lines.append( '    config.window_title = u"' + globals()["projectName"] + '"\n')
                lines.append( '    config.name = "' + globals()["projectName"] + '"\n')
                lines.append( '    config.version = "1.0"\n')
                lines.append( '    config.default_fullscreen = False\n')
                lines.append( '    config.default_text_cps = 30\n')
                f.write( ''.join(lines).encode('UTF-8') )
        except:
            REPORTERROR("Imposible escribir en " + outFile)

    def generateGlobalVariables():
        if globals()['requestedGlobalVariables'] == []: return

        LOG("Generando variables globales...")
        
        outGlobalsFile = os.path.abspath( os.path.join( globals()["projectOutDirectory"] + "/game/globals.rpy" ) )

        try:
            with open(outGlobalsFile, "w") as f:
                if len( globals()['requestedGlobalVariables'] ) > 0:
                    f.write("init python:\n")
                for var, val in globals()['requestedGlobalVariables']:
                    if isinstance(val, basestring):
                        f.write( "    " + var + " = '" + val + "'\n" )
                    else:
                        f.write( "    " + var + " = " + str(val) + "\n" )
        except:
            REPORTERROR("Imposible escribir en " + outGlobalsFile)            

    def isAnimation(animationId):
        for animation in animations:
            if animations[animation].id == animationId:
                return True
        return False

    def copyRequestedResources():
        LOG("Copiando recursos del proyecto...")

        for res in globals()["requestedResources"]:

            if isAnimation(res):
                continue

            try:
                path = os.path.abspath( globals()["projectDirectory"] + "/" + res )
                outPath = os.path.abspath( os.path.join( globals()["projectOutDirectory"], "game/" + res ) )
                if not os.path.isfile( outPath ):
                    outDir = os.path.split(outPath)[0]
                    if not os.path.exists( outDir ):
                        os.makedirs( outDir )
                    shutil.copyfile(path, outPath)
                    globals()['gccount']['Recursos'] += 1
            except:
                REPORTERROR("Falló la copia del recurso {b}" + res + "{/b}")
                return

    def generateImageDefinitions():
        LOG("Generando definiciones de imágenes...")
        
        outImagesFile = globals()["projectOutDirectory"] + "/game/images.rpy"

        try:
            with open(outImagesFile, "w") as f:
                if len( globals()["imageDefinition"] ) > 0:
                    f.write("init:\n")
                for image in globals()["imageDefinition"]:
                    f.write( "    image " + image + ' = "' + globals()["imageDefinition"][image] + '"\n' )
        except:
            REPORTERROR("Imposible escribir en " + outImagesFile)

    def requestEngineGlobals():
        requestGlobalVariable( "lector" )
        requestGlobalVariable( "save_game" )
        requestGlobalVariable( "player_score", 0 )
        requestGlobalVariable( "player_pctProgress", 0 )
        requestGlobalVariable( "player_curSection", 0 )
        requestGlobalVariable( "player_fartherVisitedSection", 0 )

        requestResource( "images/user_info/boton_info_inactivo.png" )
        requestResource( "images/user_info/boton_info_activo.png" ) 
        requestResource( "images/user_info/userinfo_bk.png" )
        requestResource( "images/user_info/userinfo_return_inactivo.png" ) 
        requestResource( "images/user_info/userinfo_return_activo.png" )
        requestResource( "images/misc/cursor_general.png" )
        requestResource( "fonts/DejaVuSans-Bold.ttf" )
        requestResource( "fonts/DejaVuSans-Oblique.ttf" )
        requestResource( "fonts/DejaVuSans.ttf" )
        requestResource( "images/misc/caja_texto.png" )
        requestResource( "images/user_info/caja.png" )
        requestResource( "sounds/sound_test.ogg" )
        requestResource( "sounds/voice_test.ogg" )

        # TODO: These resources are referenced from styles.rpy. Future versions should allow for definition of RenPy styles,
        # and this function should be discarded.

    def generationReport():
        LOG("Totales:")
        for ce in globals()['gccount']:
            LOG("\n    " + ce + ": " + str(globals()['gccount'][ce]))
        LOG("\n\n")
        
    def generateScenes():

        requestEngineGlobals()

        setupOutputDirectory()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generateCodeUtilities()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generateTransforms()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generateAnimations()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generateWordDefinitions()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generateUIScenes()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generateScriptScenes()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generateTransitionScenes()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generateConfiguration()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generateGlobalVariables()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generateImageDefinitions()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        copyRequestedResources()
        if globals()["lastError"] != None:
            return
        else:
            LOGOK()

        generationReport()

        return


