
init python:

    class ActivityScene(Scene):

        def __init__( self, xmlNode ):
            Scene.__init__(self, xmlNode)
            
            self.core = None
            self.type = None
            self.parse(xmlNode)

        def parse(self, xmlNode):
            # parse events
            for child in xmlNode:
                if child.tag == 'evento':
                    event = EventListener( child ) 
                    if event == None: continue
                    self.events[ event.id ] = event
                else:
                    self.core = self.createCore( child )

        def createCore(self, xmlNode):
            if xmlNode == None: 
                return None
            self.type = xmlNode.tag
            if xmlNode.tag == "rompecabezas":
                return self.createCorePuzzle(xmlNode)
            elif xmlNode.tag == "pregunta":
                return self.createQuestion(xmlNode)
            elif xmlNode.tag == "casillas":
                return self.createTiles(xmlNode)
            elif xmlNode.tag == "memoria":
                return self.createMemory(xmlNode)
            elif xmlNode.tag == "objetosOcultos":
                return self.createHiddenObjects(xmlNode)
            elif xmlNode.tag == "nombrarObjetos":
                return self.createNamingObjects(xmlNode)
            elif xmlNode.tag == "nombrarObjetos2":
                return self.createNamingObjects(xmlNode)
            return None

        def createCorePuzzle(self, xmlNode):

            result = {}

            result['x'] = getIntAttr(xmlNode, 'x')
            result['y'] = getIntAttr(xmlNode, 'y')
            
            result['imagePath'] = getStringAttr(xmlNode, 'imagen', None)

            result['scale'] = getFloatAttr(xmlNode, 'escala', None)

            result['rows'] = getIntAttr(xmlNode, 'filas', 2)
            result['cols'] = getIntAttr(xmlNode, 'columnas', 2)

            result['imageFreeSlot'] = getStringAttr(xmlNode, 'imagenLibre', None)

            result['deckX'] = getIntAttr(xmlNode, 'fX')
            result['deckY'] = getIntAttr(xmlNode, 'fY')
            result['deckCols'] = getIntAttr(xmlNode, 'fC', 2)

            requestResource( result['imagePath'] )
            requestResource( result['imageFreeSlot'] )

            return result

        def createQuestion(self, xmlNode):
            result = {}
            result['id'] = getStringAttr(xmlNode, 'id', None)
            result['texto'] = getStringAttr(xmlNode, 'texto', None)

            result['respuestas'] = []
            for child in xmlNode:
                if child.tag == 'respuesta':
                    respuesta = {}
                    respuesta['id'] = getStringAttr(child, 'id', None)
                    respuesta['texto'] = getStringAttr(child, 'texto', None)
                    result['respuestas'].append( respuesta )

            return result
            
        def createTiles(self, xmlNode):
            result = {}
            result['x'] = getIntAttr(xmlNode, 'x', None)
            result['y'] = getIntAttr(xmlNode, 'y', None)
            result['casillas'] = []
            result['fichas'] = []
            for child in xmlNode:
                if child.tag == 'mensajeError':
                    result['mensajeErrorFuente'] = getStringAttr(child, 'fuente', None)
                    result['mensajeErrorTexto'] = getStringAttr(child, 'texto', None)
                    requestResource( result['mensajeErrorFuente'] )
                elif child.tag == 'casilla':
                    casilla = {}
                    casilla['id'] = getStringAttr(child, 'id', None)
                    casilla['fuente'] = getStringAttr(child, 'fuente', None)
                    casilla['x'] = getIntAttr(child, 'x')
                    casilla['y'] = getIntAttr(child, 'y')
                    casilla['ancho'] = getIntAttr(child, 'ancho')
                    casilla['alto'] = getIntAttr(child, 'alto')
                    result['casillas'].append( casilla )
                    requestResource( casilla['fuente'] )
                elif child.tag == 'ficha':
                    ficha = {}
                    ficha['id'] = getStringAttr(child, 'id', None)
                    ficha['fuente'] = getStringAttr(child, 'fuente', None)
                    ficha['x'] = getIntAttr(child, 'x')
                    ficha['y'] = getIntAttr(child, 'y')
                    ficha['ancho'] = getIntAttr(child, 'ancho')
                    ficha['alto'] = getIntAttr(child, 'alto')
                    result['fichas'].append( ficha )
                    requestResource( ficha['fuente'] )

            return result

        def createMemory(self, xmlNode):
            result = {}
            result['x'] = getIntAttr(xmlNode, 'x', None)
            result['y'] = getIntAttr(xmlNode, 'y', None)
            result['margen'] = getIntAttr(xmlNode, 'margen', None)
            result['maximoHorizontal'] = getIntAttr(xmlNode, 'maximoHorizontal', None)
            result['cartas'] = []
            for child in xmlNode:
                if child.tag == 'carta':
                    carta = {}
                    carta['fuente'] = getStringAttr(child, 'fuente')
                    carta['base'] = getStringAttr(child, 'base')
                    carta['ancho'] = getIntAttr(child, 'ancho')
                    carta['alto'] = getIntAttr(child, 'alto')
                    carta['valor'] = getIntAttr(child, 'valor')
                    result['cartas'].append( carta )
                    requestResource( carta['fuente'] )
                    requestResource( carta['base'] )

            return result

        def createHiddenObjects(self, xmlNode):
            result = {}
            result['fuenteListaObjetos'] = getStringAttr(xmlNode, 'fuenteListaObjetos', None)
            result['fuenteClickAleatorio'] = getStringAttr(xmlNode, 'fuenteClickAleatorio', None)
            result['objetos'] = []
            for child in xmlNode:
                if child.tag == 'objeto':
                    objeto = {}
                    objeto['fuente'] = getStringAttr(child, 'fuente')
                    objeto['nombre'] = getStringAttr(child, 'nombre')
                    objeto['x'] = getIntAttr(child, 'x')
                    objeto['y'] = getIntAttr(child, 'y')
                    objeto['ancho'] = getIntAttr(child, 'ancho')
                    objeto['alto'] = getIntAttr(child, 'alto')
                    result['objetos'].append( objeto )
                    requestResource( objeto['fuente'] )

            return result

        def createNamingObjects(self, xmlNode):
            result = {}
            result['objetos'] = []
            for child in xmlNode:
                if child.tag == 'objeto':
                    objeto = {}
                    objeto['fuente'] = getStringAttr(child, 'fuente')
                    objeto['nombre'] = getStringAttr(child, 'nombre')
                    objeto['x'] = getIntAttr(child, 'x')
                    objeto['y'] = getIntAttr(child, 'y')
                    objeto['ancho'] = getIntAttr(child, 'ancho')
                    objeto['alto'] = getIntAttr(child, 'alto')
                    result['objetos'].append( objeto )
                    requestResource( objeto['fuente'] )

            return result

        def generate(self):
            f = []

            if self.image != "" and self.image != None:
                requestImageDefinition( self.id + "_bg", self.image )
                requestResource( self.image )
                writeLine(f, "scene " + self.id + "_bg", 1)
            else:
                writeLine(f, "scene black", 1)

            for event in self.events:
                self.events[ event ].generateHandler(f)

            if self.type == 'rompecabezas':
                self.generatePuzzle(f)
            elif self.type == 'pregunta':
                self.generateQuestion(f)
            elif self.type == 'casillas':
                self.generateTiles(f)
            elif self.type == 'memoria':
                self.generateMemory(f)
            elif self.type == 'objetosOcultos':
                self.generateHiddenObjects(f)
            elif self.type == 'nombrarObjetos':
                self.generateNamingObjects(f)
            elif self.type == 'nombrarObjetos2':
                self.generateNamingObjects2(f)
            return f

        def generateNamingObjects(self, f):
            writeLine(f, "python:", 1)

            self.generateNamingObjectsScreens()

            out = "elements = ["
            for objeto in self.core['objetos']:
                out = out + '"' + objeto['nombre'] + '", '
            out = out[:-2]
            out = out + "]"
            writeLine(f, out, 2)
            writeLine(f, "activityVisualIdentification(elements)", 2)


        def generateNamingObjects2(self, f):
            writeLine(f, "python:", 1)
            self.generateNamingObjectsScreens()

            out = "elements = ["
            for objeto in self.core['objetos']:
                out = out + '"' + objeto['nombre'] + '", '
            out = out[:-2]
            out = out + "]"
            writeLine(f, out, 2)
            writeLine(f, "activityVisualIdentification2(elements)", 2)

        def generateNamingObjectsScreens(self):
            writeLine(f, "screen " + self.id + ":")
            writeLine(f, "")
            for objeto in self.core['objetos']:
                writeLine(f, 'add "' + objeto['fuente'] + '" pos(' + str(objeto['x']) + ', ' + str(objeto['y']) + ')', 1)
                writeLine(f, "")

        def generateHiddenObjects(self, f):
            writeLine(f, "python:", 1)
            writeLine(f, self.id + " = CHiddenObjectScene()", 2)
            for objeto in self.core['objetos']:
                writeLine(f, objeto['nombre'] + ' = CHiddenObject(' + self.id + ', "' + objeto['fuente'] + '", Text("' + objeto['nombre'] + '"), ' + str(objeto['x']) + ', ' + str(objeto['y']) + ', ' + str(objeto['ancho']) + ', ' + str(objeto['alto']) + ')', 2)
            for objeto in self.core['objetos']:
                writeLine(f, self.id + ".append(" + objeto['nombre'] + ", 2, 2, 50)", 2)
            writeLine(f, self.id + '.addFraudImage("' + self.core['fuenteClickAleatorio'] + '")', 2)
            writeLine(f, self.id + '.addObjectFrame("' + self.core['fuenteListaObjetos'] + '")', 2)
            writeLine(f, "ui.add(" + self.id + ")", 2)
            writeLine(f, "ui.interact()", 2)


        def generateMemory(self, f):
            writeLine(f, "python:", 1)
            writeLine(f, self.id + " = CCards()", 2)
            for carta in self.core['cartas']:
                writeLine(f, self.id + '.addCardPair("' + carta['fuente'] + '", "' + carta['base'] + '", ' + str(carta['ancho']) + ', ' + str(carta['alto']) + ', ' + str(carta['valor']) + ')', 2)
            writeLine(f, self.id + ".offx = " + str(self.core['x']), 2)
            writeLine(f, self.id + ".offy = " + str(self.core['y']), 2)
            writeLine(f, self.id + ".xgap = " + str(self.core['margen']), 2)
            writeLine(f, self.id + ".max_xcord = " + str(self.core['maximoHorizontal']), 2)
            writeLine(f, self.id + ".shuffle()", 2)
            writeLine(f, "ui.add(" + self.id + ")", 2)
            writeLine(f, "ui.interact()", 2)

        def generateTiles(self, f):
            writeLine(f, "python:", 1)
            writeLine(f, self.id + " = CDrag()", 2)
            for casilla in self.core['casillas']:
                writeLine(f, self.id + ".addSlot(" + casilla['id'] + ', ' + str(casilla['x']) + ', ' + str(casilla['y']) + ', ' + str(casilla['ancho']) + ', ' + str(casilla['alto']) + \
                ', "' + casilla['fuente'] + '")', 2)
            for ficha in self.core['fichas']:
                writeLine(f, self.id + ".addPiece(" + ficha['id'] + ', ' + str(ficha['x']) + ', ' + str(ficha['y']) + ', ' + str(ficha['ancho']) + ', ' + str(ficha['alto']) + \
                ', "' + ficha['fuente'] + '")', 2)
            if 'mensajeErrorFuente' in self.core:
                writeLine(f, self.id + ".addWrongMsg('" + self.core['mensajeErrorFuente'] + "')", 2)
            writeLine(f, "ui.add(" + self.id + ")", 2)
            writeLine(f, "ui.interact()", 2)

        def generateQuestion(self, f):
            writeLine(f, "python:", 1)
            writeLine(f, 'answerCode = activityAsk("' + self.core['id'] + '", "' + self.core['texto'] + '")', 2)
            writeLine(f, 'activityResult(answerCode, "' + self.core['respuestas'][0]['texto'] + '", "' + self.core['respuestas'][1]['texto'] + '")', 2)

        def generatePuzzle(self, f):
            writeLine(f, "python:", 1)
            writeLine(f, "cpuzzle = CPuzzle(" + str(self.core['x']) + ", " + str(self.core['y']) + ")", 2)
            if self.core['imagePath'] != None:
                requestResource( self.core["imagePath"] )
                writeLine(f, "cpuzzle.setImage('" + self.core['imagePath'] + "')", 2)
            if self.core['scale'] != None:
                writeLine(f, "cpuzzle.scaleImage(" + str(self.core['scale']) + ")", 2)
            writeLine(f, "cpuzzle.setupBoard(" + str(self.core['rows']) + ", " + str(self.core['cols']) + ")", 2)
            if self.core['imageFreeSlot'] != None:
                requestResource( self.core["imageFreeSlot"] )
                writeLine(f, "cpuzzle.setImageEmptySlot('" + self.core['imageFreeSlot'] + "')", 2)
            writeLine(f, "cpuzzle.setupPieces(" + str(self.core['deckX']) + ", " + str(self.core['deckY']) + ", " + str(self.core['deckCols']) + ")", 2)

            writeLine(f, "ui.add(cpuzzle)", 2)
            writeLine(f, "ui.interact()", 2)
