
init python:

    import re

    class EventListener(object):

        def __init__(self, xmlNode ):
        
            # <evento id="click">...

            self.id = xmlNode.attrib.get('id')

            self.parsingTable = {
                'mostrarTexto' : [ ('x',TXMLInt), ('y',TXMLInt), ('texto',TXMLStr), ('ancho', TXMLInt), ('flotante',TXMLInt), ('imagen',TXMLStr), ('imx',TXMLInt), ('imy',TXMLInt) ],
                'mostrarImagen' : [ ('id', TXMLStr), ('fuente',TXMLStr), ('x',TXMLInt), ('y',TXMLInt), ('transformacion', TXMLStr) ],
                'reproducirSonido' : [ ('fuente',TXMLStr) ],
                'reproducirMusica' : [ ('fuente',TXMLStr) ],
                'reproducirVoz' : [ ('fuente',TXMLStr) ],
                'ir' : [ ('id', TXMLStr), ('escena',TXMLStr) ],
                'entradaTexto' : [ ('id', TXMLStr), ('texto',TXMLStr), ('longitud',TXMLStr) ],
                'copiarVariable' : [ ('id', TXMLStr), ('destino', TXMLStr), ('origen',TXMLStr), ('modificador', TXMLStr) ],
                'esperar' : [ ('id', TXMLStr), ('t', TXMLInt) ],
                'ocultarImagen' : [ ('id', TXMLStr), ('imagen', TXMLStr) ],
                'entrarSeccion' : [ ('id', TXMLStr), ('seccion', TXMLInt) ],
                'controlInformacionProgreso' : [ ('id', TXMLStr), ('mostrar', TXMLInt) ]
            }

            self.actions = []

            for child in xmlNode:
                action = parseAttributes( child, self.parsingTable )
                if action != {}:
                    self.actions.append ( (child.tag, action) )


        def formActionParameters(self, actionName, actionAttributes):
            
            attributes = ""                    

            parsingEntities = self.parsingTable[actionName]
            for entity, entityType in parsingEntities:
                if entityType != TXMLStr:
                    attributes = attributes + entity + "=" + str(actionAttributes[entity]) + ", "
                else:
                    texto = actionAttributes[entity]

                    if texto in ["", None]: continue
                    
                    # Sustitución y Verificación de variables globales dentro de cadena de texto. Si una variable global
                    # en el texto no está en la lista global de variables, el texto no se genera.
                    texto = re.sub("%\((\S+)\)", '" + str(globals()["\g<1>"]) + "', texto)
                    fa = re.findall("%\((\S+)\)", actionAttributes[entity])
#                    globalVariableNotRequested = False
#                    if fa != []:
#                        for var in fa:
#                            if not var in globals()["requestedGlobalVariables"]:
#                                globalVariableNotRequested = True
#                                break
#                    if globalVariableNotRequested:
#                        continue
                    ###

                    attributes = attributes + entity + '="' + texto + '", '
            
            # remove last ',' and space
            attributes = attributes[:-2]
            
            return attributes

        def generateHandler(self, f):

            if self.id == 'entrar':
                self.generateOnEnterHandler(f)

        def generateOnEnterHandler(self, f):

            for actionName, actionAttributes in self.actions:

                if not actionName in self.parsingTable: 
                    continue

                actionLine = None

                if actionName == "mostrarTexto":
                    parameterList = self.formActionParameters(actionName, actionAttributes)
                    actionLine = "actionShowText(" + parameterList + ")"
                    writeLine(f, "$ " + actionLine, 1)

                elif actionName == "mostrarImagen":

                    fuente = actionAttributes["fuente"]
                    if not isAnimation(fuente):
                        if actionAttributes["id"] == None:
                            continue
                        fuente = actionAttributes['id']
                        requestImageDefinition( fuente, actionAttributes['fuente'] )

                    if actionAttributes['transformacion'] == None:
                        actionLine = "show " + fuente + ":"
                        writeLine(f, actionLine, 1)
                        writeLine(f, " pos(" + str(actionAttributes['x']) + ", " + str(actionAttributes['y']) + ")", 2)
                    else:
                        actionLine = "show " + fuente + " at " + actionAttributes['transformacion']
                        writeLine(f, actionLine, 1)
                    requestResource( actionAttributes['fuente'] )

                elif actionName == "reproducirSonido":
                    actionLine = "actionPlaySound('" + actionAttributes['fuente'] + "')"
                    writeLine(f, "$ " + actionLine, 1)
                    requestResource( actionAttributes['fuente'] )

                elif actionName == "reproducirMusica":
                    actionLine = "actionPlayMusic('" + actionAttributes['fuente'] + "')"
                    writeLine(f, "$ " + actionLine, 1)
                    requestResource( actionAttributes['fuente'] )

                elif actionName == "reproducirVoz":
                    # Voice isn't a traditional channel, like sound or music. It has some additional logic (so the 
                    # voices end when the users click past them). Use "voice" rather than "play voice".
                    actionLine = 'voice "' + actionAttributes["fuente"] + '"'
                    writeLine(f, actionLine, 1)
                    requestResource( actionAttributes['fuente'] )

                elif actionName == "entradaTexto":
                    if actionAttributes['id'] == None: continue
                    texto = ""
                    if actionAttributes['texto'] != None:
                        texto = actionAttributes['texto']

                    writeLine(f, "python:", 1)
                    writeLine(f, actionAttributes['id'] + ' = ""', 2)
                    writeLine(f, 'while ' + actionAttributes['id'] + ' == "":', 2)
                    writeLine(f, actionAttributes['id'] + ' = renpy.input("' + texto + '", "", None, "{}", 30) or ""', 3)

                elif actionName == "copiarVariable":
                    if actionAttributes['origen'] == None or actionAttributes['destino'] == None: continue

                    #requestGlobalVariable( actionAttributes['destino'] )

                    if actionAttributes['modificador'] in [None, ""]:
                        writeLine(f, "$ " + actionAttributes['destino'] + " = " + actionAttributes['origen'], 1)
                    elif actionAttributes['modificador'] == 'titulo':
                        writeLine(f, "$ " + actionAttributes['destino'] + " = " + actionAttributes['origen'] + ".title()", 1)

                elif actionName == "esperar":
                    t = 0
                    if actionAttributes['t'] != None:
                        t = actionAttributes['t']

                    writeLine(f, "$ ui.timer(" + str(t) + ", ui.returns(1))", 1)
                    writeLine(f, "$ ui.interact()", 1)

                elif actionName == "ocultarImagen":
                    if actionAttributes['imagen'] == None: 
                        continue

                    writeLine(f, "hide " + actionAttributes['imagen'], 1)

                elif actionName == "entrarSeccion":
                    if actionAttributes['seccion'] == None: 
                        continue

                    writeLine(f, "$ enterSection(" + str(actionAttributes['seccion']) + ")", 1)


                elif actionName == "controlInformacionProgreso":
                    if actionAttributes['mostrar'] == None: 
                        continue

                    writeLine(f, "$ controlProgress(" + str(actionAttributes['mostrar']) + ")", 1)
