

init python:
    
    #############################################################
    # CDrag
    # 
    #############################################################
    class CDrag(renpy.Displayable):

        #############################################################
        def __init__(self, **kwargs):

            renpy.Displayable.__init__(self, **kwargs)
                       
            # listen to events?
            self.sensitive = True
            
            # drag slots
            self.slots = []
            
            # drag pieces
            self.pieces = []
            
            # puzzle piece currently being dragged
            self.draggingPiece = None
            
            # wrong meesage
            self.showWrongMsg = False
            self.wrongMsg = None
            
        #############################################################
        def show(self, layer='master'):
            pass            
            
        #############################################################
        def hide(self, layer='master'):
            pass
        
        #############################################################
        def addSlot(self, sindex, sx, sy, swidth, sheight, sface):
            self.slots.append( dict(index=sindex, x=sx, y=sy, rect=(sx, sy, swidth, sheight), face=renpy.easy.displayable_or_none(sface), piece=None) )

        #############################################################
        def addPiece(self, psindex, px, py, pwidth, pheight, pface):
            self.pieces.append( dict(x=px, y=py, width=pwidth, height=pheight, face=renpy.easy.displayable_or_none(pface), solutionIndex=psindex, curSlot=None) )
            
        #############################################################
        def addWrongMsg(self, wmFace):
            self.wrongMsg = renpy.easy.displayable_or_none(wmFace)

        #############################################################
        def getSlot(self, x, y):
            
            # compute cell (row,col) for point (x,y). A (-1-1) value implies an out-of-board point.

            for slot in self.slots:
                if inRect(x, y, slot['rect']):
                    return slot
            
            return None
        
        #############################################################            
        def boardCompleted(self):
            for slot in self.slots:
                if slot['piece'] == None:
                    return 0
                if slot['piece']['solutionIndex'] != slot['index']:
                    return 0
            return 1
        
        #############################################################        
        def checkSolution(self):
            result = self.boardCompleted()
            if result == 1:
                self.sensitive = False
                return result

            for slot in self.slots:
                if slot['piece'] == None:
                    return result
            
            # si llegamos acá, todas las piezas están colocadas, pero hay al menos dos fuera de lugar
            self.showWrongMsg = True
            pygame.time.set_timer(pygame.USEREVENT + 1, 4000)
            self.sensitive = False
            
            return result
                
        #############################################################
        def getPieceRect(self, piece):
            return (piece['x'], piece['y'], piece['width'], piece['height'])
            #return (piece['x'], piece['y'], piece['x'] + piece['width'], piece['y'] + piece['height'])

        #############################################################
        def render(self, width, height, st, at):

            # get render object
            rv = renpy.Render(width, height)

            # render slots
            for slot in self.slots:
                slotSurf = renpy.render(slot['face'], width, height, st, at)
                rv.blit(slotSurf, (slot['x'], slot['y']))

            # render pieces
            for piece in self.pieces:
                if piece != self.draggingPiece:
                    pieceSurf = renpy.render(piece['face'], width, height, st, at)
                    rv.blit(pieceSurf, (piece['x'], piece['y']))

            # La pieza que está actualmente seleccionada (si alguna lo está) se dibuja de última para
            # garantizar su sobreposición                        
            if self.draggingPiece != None:
                pieceSurf = renpy.render(self.draggingPiece['face'], width, height, st, at)
                rv.blit(pieceSurf, (self.draggingPiece['x'], self.draggingPiece['y']))
            
            if self.showWrongMsg == True and self.wrongMsg != None:
                msgSurf = renpy.render(self.wrongMsg, width, height, st, at)
                rv.blit(msgSurf, (0, 20))
                          
            return rv
        
        #############################################################
        def event(self, ev, x, y, st):           

            if ev.type == pygame.USEREVENT + 1:
                pygame.time.set_timer(pygame.USEREVENT + 1, 0)
                self.showWrongMsg = False
                self.sensitive = True
                renpy.redraw(self,0)

            if not self.sensitive:
                return

            if ev.type == pygame.MOUSEBUTTONDOWN and ev.button == 1:
                
                # debug return
                #return 1
                
                if self.draggingPiece == None:
                    # La inspección de la pieza seleccionada debe hacerse en orden inverso al dibujado, porque las
                    # piezas en posiciones superiores (on-top) tienen prioridad para detección de eventos click.
                    # Es decir, la pieza que se dibuja de última (está al final de la lista de piezas) tiene mayor
                    # prioridad para detección de click.
                    #for c in self.pieces:
                    for c in list(reversed(self.pieces)):
                        if inRect(x, y, self.getPieceRect(c)):
                            self.pieces.remove(c)
                            self.pieces.append(c)
                            self.draggingPiece = c
                            self.draggingPiece['x'] = x - (self.draggingPiece['width'] / 2.0)
                            self.draggingPiece['y'] = y - (self.draggingPiece['height'] / 2.0)
                            if self.draggingPiece['curSlot'] != None:
                                self.draggingPiece['curSlot']['piece'] = None
                                self.draggingPiece['curSlot'] = None
                            renpy.redraw(self,0)
                            break
                else:                    
                    slot = self.getSlot(x, y)
                    if slot != None:
                        self.draggingPiece['x'] = slot['x'] + 29 - self.draggingPiece['width'] / 2.0
                        self.draggingPiece['y'] = slot['y'] + 29 - self.draggingPiece['height'] / 2.0
                        self.draggingPiece['curSlot'] = slot
                        renpy.music.play('sounds/sonido_logro.ogg', channel='sound', fadeout=0)
                        if slot['piece'] != None:
                            tmp = slot['piece']
                            slot['piece'] = self.draggingPiece
                            self.draggingPiece = tmp
                            self.draggingPiece['curSlot'] = None      
                            self.draggingPiece['x'] = x - (self.draggingPiece['width'] / 2.0)
                            self.draggingPiece['y'] = y - (self.draggingPiece['height'] / 2.0)
                        else:
                            slot['piece'] = self.draggingPiece
                            self.draggingPiece = None
                            #check if puzzle solved
                            solved = self.checkSolution()
                            if solved == 1:
                                return 1
                    else:
                        self.draggingPiece = None
                    renpy.redraw(self,0)

            if (ev.type == pygame.MOUSEMOTION) and (self.draggingPiece != None):
                self.draggingPiece['x'] = x - (self.draggingPiece['width'] / 2.0)
                self.draggingPiece['y'] = y - (self.draggingPiece['height'] / 2.0)
                renpy.redraw(self,0)

    #############################################################
    #############################################################
    #############################################################
