# `Maestro Virtual' una novela visual que relata el 
# cuento `El Principe Feliz' de Oscar Wilde.
# Copyright (C) 2010 Fundación CENDITEL.

# La Fundación CENDITEL concede permiso para copiar, distribuir 
# y/o modificar este programa bajo los términos de la licencia de
# software GPL versión 2.0 de la Free Software Foundation. 

# Este programa se distribuye con la esperanza de que sea útil, 
# pero SIN NINGUNA GARANTÍA; tampoco las implícitas garantías
# de MERCANTILIDAD o ADECUACIÓN A UN PROPÓSITO PARTICULAR. 
# Consulte la licencia GPL para más detalles.

# Una copia de la licencia en inglés y en español puede obtenerse 
# en los siguientes sitios en Internet:
# En inglés: http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# En español: http://www.es.gnu.org/modules/content/index.php?id=8

init python:
    #################################################################
    import math
    import random

    # initialize random numbers
    random.seed()

    # recording particle creation
    creationIndex = 0

    #################################################################
    # Ring particles
    # ----------------
    def RingParticles(image, center_x=100, center_y=100, amplitude=50, max_particles=40, angular_speed=150, **kwargs):
        return Particles(RingFactory(image, max_particles, angular_speed, amplitude, center_x, center_y, **kwargs))

    # ---------------------------------------------------------------
    class RingFactory(object):
        def __init__(self, image, max_particles, angular_speed, amplitude, center_x, center_y, **kwargs):

            self.max_particles = max_particles

            self.angular_speed = angular_speed

            self.amplitude = amplitude

            self.center_x = center_x
            self.center_y = center_y

            # initialize the images
            self.image = self.image_init(image)


        def create(self, particles, st):
            """
            This is internally called every frame by the Particles object to create new particles.
            We'll just create new particles if the number of particles on the screen is
            lower than the max number of particles we can have.
            """
            # if we can create a new particle...
            if particles is None or len(particles) < self.max_particles:

                return [ RingParticle(self.image[0],      # the image used by the particle
                                      self.angular_speed,
                                      self.amplitude,
                                      self.center_x,
                                      self.center_y
                                      ) ]

        def image_init(self, image):
            rv = []
            rv.append( im.FactorScale(image, 0.5))
                
            return rv


        def predict(self):
            """
            This is called internally by the Particles object to predict the images the particles
            are using. It's expected to return a list of images to predict.
            """
            return self.image

    # ---------------------------------------------------------------
    class RingParticle(object):
        """
        Represents every particle in the screen.
        """
        def __init__(self, image, angular_speed, amplitude, center_x, center_y):

            # The image used by this particle
            self.image = im.MatrixColor(image, im.matrix.tint(1.0,random.randint(1,100)/100.0,random.randint(1,100)/100.0))
            #~ self.image = image

            # For safety (and since we don't have snow going from the floor to the sky o.o)
            # if the vertical speed of the particle is lower than 1, we use 1.
            # This prevents the particles of being stuck in the screen forever and not falling at all.
            if angular_speed <= 0:
                angular_speed = 1

            self.amplitude = amplitude

            # particle's speed
            self.angular_speed = angular_speed

            # The last time when this particle was updated (used to calculate the unexpected delay
            # between updates, aka lag)
            self.oldst = None

            # the horizontal/vertical positions of this particle
            self.center_x = center_x
            self.center_y = center_y

            self.amplitude = amplitude

            global creationIndex
            self.acumtime = creationIndex
            creationIndex = creationIndex + 1

            #~self.ttl = random.randint(1,10) / 2.0


        def update(self, st):
            """
            Called internally in every frame to update the particle.
            """

            # calculate lag
            if self.oldst is None:
                self.oldst = st

            lag = st - self.oldst
            self.oldst = st

            # update the position
            self.acumtime = self.acumtime + lag
            self.xpos = self.center_x + self.amplitude * math.cos(self.angular_speed * self.acumtime)
            self.ypos = self.center_y + self.amplitude * math.sin(self.angular_speed * self.acumtime)

            # verify if the particle went out of the screen so we can destroy it.
            #~if self.ypos > renpy.config.screen_height or\
               #~(self.wind< 0 and self.xpos < 0) or (self.wind > 0 and self.xpos > renpy.config.screen_width):
                #~return None

            #~if self.acumtime > self.ttl:
                #~return None

            # returns the particle as a Tuple (xpos, ypos, time, image)
            # since it expects horizontal and vertical positions to be integers, we have to convert
            # it (internal positions use float for smooth movements =D)
            return int(self.xpos), int(self.ypos), st, self.image

