
init python:

    import codecs

    class UIScene(Scene):

        def __init__( self, xmlNode ):
            Scene.__init__(self, xmlNode)

            self.widgets = []
            self.parseWidgets(xmlNode)

            self.sceneTranslate = {
                'menuprincipal':'main_menu',
                'cargar':'load',
                'salvar':'save',
                'opciones':'preferences',
                'acerca':'about'
            }

        def parseWidgets(self, xmlNode):
            for child in xmlNode:
                widget = Widget(child)
                self.widgets.append( widget )

        def parse(self, xmlNode):
            # parse events
            events = []
            for child in xmlNode:
                if child.tag == "evento":
                    events.append( self.parseEvent(child) )
            return events

        def parseEvent(self, xmlNode):
            eventID = getStringAttr(xmlNode, "id")

            for child in xmlNode:
                if child.tag == "acciones":
                    return (eventID, self.parseActions(child))

            return None

        def parseActions(self, xmlNode):
            actions = []
            attributes = None
            for child in xmlNode:
                attributes = self.parseActionAttributes(child)
                actions.append( (child.tag, attributes) )

            return actions

        def parseActionAttributes(self, xmlNode):
            attributes = {}
            
            if not xmlNode.tag in self.parsingTable:
                return {}

            parsingEntities = self.parsingTable[ xmlNode.tag ]
            for entity, entityType in parsingEntities:
                if entityType == TXMLInt:
                    attributes[entity] = getIntAttr(xmlNode, entity, None)
                elif entityType == TXMLFloat:
                    attributes[entity] = getFloatAttr(xmlNode, entity, None)
                elif entityType == TXMLStr:
                    attributes[entity] = getStringAttr(xmlNode, entity, None)

            return attributes

        def generate(self):
            f = ["\n"]
            #if self.id != "save" and self.id != "load":
                
            screenid = self.id
            if screenid in self.sceneTranslate:
                screenid = self.sceneTranslate[screenid]
            writeLine(f, "screen " + screenid + ":")
            writeLine(f, "")

            if self.id in self.sceneTranslate:
                writeLine(f, "tag menuscreen", 1)

            if self.id in ["infoLogros", "infoPuntos", "infoMenu"]:
                writeLine(f, "tag user_info_tab", 1)
                writeLine(f, "modal True", 1)

            if self.image != None:
                requestResource( self.image )
                writeLine(f, 'add "' + self.image + '"', 1)


            #if self.id == "options":
            #    writeLine(f, 'grid 2 1:', 1)
            #    writeLine(f, 'align (0.5, 0.48)', 2)


            
            if self.id == "opciones":

                writeLine(f, 'grid 2 1:', 1)
                writeLine(f, 'style_group "prefs"', 2)
                writeLine(f, 'align (0.5, 0.48)', 2)

                vbox_config_count = 0
                for widget in self.widgets:
                    if widget.type == Widget.CONFIGURATION:
                        # group in 3 vboxes
                        if vbox_config_count == 0:
                            writeLine(f, 'vbox:', 2)
                        vbox_config_count += 1
                        if vbox_config_count == 3:
                            vbox_config_count = 0
                        self.generateWidget(f, widget)

            elif self.id == "cargar":
                fin = os.path.abspath( os.path.join( renpy.loader.transfn(""), "core/basic_load") )
                #with codecs.open("basic_load", "r", "utf-8") as fileBasicLoad:
                with codecs.open(fin, "r", "utf-8") as fileBasicLoad:
                    load_code = fileBasicLoad.read()
                writeLine(f, load_code)

            elif self.id == "salvar":
                fin = os.path.abspath( os.path.join( renpy.loader.transfn(""), "core/basic_save") )
                #with codecs.open("basic_save", "r", "utf-8") as fileBasicSave:
                with codecs.open(fin, "r", "utf-8") as fileBasicSave:
                    save_code = fileBasicSave.read() 
                writeLine(f, save_code)

            for widget in self.widgets:
                if widget.type != Widget.CONFIGURATION:
                    self.generateWidget(f, widget)

            return f

        def generateWidget(self, f, widget):
            if widget.type == Widget.BUTTON:

                action = "Quit()"

                for event in widget.events:
                    for actionName, actionAttrib in widget.events[event].actions:
                        if actionName == "ir":
                            escena = actionAttrib['escena']
                            if escena == None:
                                continue
                            if escena in ['cargar', 'salvar', 'opciones', 'acerca']:
                                action = 'ShowMenu("' + self.sceneTranslate[escena] + '")'
                            elif escena == 'salir':
                                action = "Quit()"
                            elif escena == 'inicio':
                                action = 'Start()'
                            elif escena == 'previa':
                                action = 'Return()'
                            elif escena == 'previaProgreso':
                                action = 'Hide("user_info_tab")'
                            elif escena in ['infoPuntos', 'infoLogros', 'infoMenu']:
                                action = 'Show("' + escena + '")'
                            else:
                                action = 'Jump("' + escena + '")'
                writeLine(f, 'imagebutton idle "' + widget.attributes['fuente'] + '" hover "' + widget.attributes['fuenteResaltado'] + '" pos(' + str(widget.attributes['x']) + ', ' + str(widget.attributes['y']) + ') ' + 'action ' + action, 1)
                requestResource( widget.attributes['fuente'] )
                requestResource( widget.attributes['fuenteResaltado'] )
            elif widget.type == Widget.LABEL:

                extra = ''
                if widget.attributes['extra'] != None:
                    extra = ' ' + widget.attributes['extra']

                if widget.attributes['datos'] in [None, ""] and not widget.attributes['texto'] in [None, ""]:
                    writeLine(f, 'text "' + widget.attributes['texto'] + '" pos(' + str(widget.attributes['x']) + ', ' + str(widget.attributes['y']) + ')' + extra, 1)
                elif not widget.attributes['datos'] in [None, ""]:
                    writeLine(f, 'text str(globals()["' + widget.attributes['datos'] + '"]) pos(' + str(widget.attributes['x']) + ', ' + str(widget.attributes['y']) + ')' + extra, 1)

            elif widget.type == Widget.IMAGE:

                s = 'add "' + widget.attributes['fuente'] + '" pos(' + str(widget.attributes['x']) + ', ' + str(widget.attributes['y']) + ')'
                if widget.attributes['transformacion'] != None:
                
                # TODO
                # Check if transformation exists
                    s = s + ' at ' + widget.attributes['transformacion']

                writeLine(f, s, 1)


                requestResource( widget.attributes['fuente'] )
            elif widget.type == Widget.MUSIC:
                s = 'on "show" action Play("music", "' + widget.attributes['fuente'] + '")'
                writeLine(f, s, 1)
                requestResource( widget.attributes['fuente'] )
            elif widget.type == Widget.IMAGEHORIZONTALSLICE:
                # obtener estos valores a partir del análisis de la fuente, y considerar que pueden ir multiplicados por datos
                cropWidth = 10
                cropHeight = 10
                writeLine(f, 'add "' + widget.attributes['fuente'] + '" crop(' + str(widget.attributes['x']) + ', ' + str(widget.attributes['y']) + ', ' + str(cropWidth) + ', ' + str(cropHeight) + ')', 1)
            elif widget.type == Widget.SECTIONSELECTOR:
                writeLine(f, "for i in range(10):", 1)
                writeLine(f, '$ x = ' + str(widget.attributes['x']) + ' + (i % ' + str(widget.attributes['columnas']) + ') * 50', 2)
                writeLine(f, '$ y = ' + str(widget.attributes['y']) + ' + (i / ' + str(widget.attributes['columnas']) + ') * 50', 2)
                writeLine(f, 'imagebutton idle "' + widget.attributes['fuente'] + '" hover "' + widget.attributes['fuenteResaltado'] + '" :', 2)
                writeLine(f, 'pos(x, y)', 3)
                writeLine(f, 'add Text(str(i+1)):', 2)
                writeLine(f, 'align(0.5, 0.5)', 3)
                writeLine(f, 'pos(x+15, y+15)', 3)
            elif widget.type == Widget.TEXTAREA:
                writeLine(f, 'frame:', 1)
                writeLine(f, 'xpadding 10', 2)
                writeLine(f, 'ypadding 20', 2)
                writeLine(f, 'align (0.4, 0.6)', 2)
                writeLine(f, 'side "t c r":', 2)
                writeLine(f, 'area (0, 0, 710, 300)', 3)
                writeLine(f, 'label "' + globals()["projectName"] + '"' , 3)
                writeLine(f, 'viewport id "vpabout":', 3)
                writeLine(f, 'vbox:', 4)
                for line, extra in widget.lines:
                    texto = ""
                    textoextra = ""
                    if line != None: texto = line
                    if extra != None: textoextra = ' ' + extra
                    writeLine(f, "text '" + texto + "'" + textoextra, 5)
                writeLine(f, 'vbar value YScrollValue("vpabout")', 3)
            elif widget.type == widget.CONFIGURATION:
                writeLine(f, 'frame:', 3)
                writeLine(f, 'area (0, 0, 300, 120)', 4)
                if widget.id == "pantalla":
                    writeLine(f, 'style "framedisplay"', 4)
                    writeLine(f, 'style_group "prefs"', 4)
                    writeLine(f, 'has vbox', 4)
                    name = "Pantalla"
                    writeLine(f, 'label "' + name + '"', 4)
                    writeLine(f, 'textbutton _("Ventana") action Preference("display", "window")', 4)
                    writeLine(f, 'textbutton _("Pantalla Completa") action Preference("display", "fullscreen")', 4)
                elif widget.id == "velocidadTexto":
                    writeLine(f, 'style "frametext"', 4)
                    writeLine(f, 'style_group "prefs"', 4)
                    writeLine(f, 'has vbox', 4)
                    name = "Velocidad del Texto"
                    writeLine(f, 'label "' + name + '"', 4)
                    writeLine(f, 'bar value Preference("text speed")', 4)
                elif widget.id == "actividades":
                    writeLine(f, 'style "frameact"', 4)
                    writeLine(f, 'style_group "prefs"', 4)
                    writeLine(f, 'has vbox', 4)
                    name = "Actividades"
                    writeLine(f, 'label "' + name + '"', 4)
                    writeLine(f, 'textbutton "Todas las actividades" action ToggleVariable("player_sageMode", 0) style "soundtest_button"', 4)
                    writeLine(f, 'textbutton "Sin preguntas abiertas" action ToggleVariable("player_sageMode", 1) style "soundtest_button"', 4)
                    writeLine(f, 'textbutton "Sin actividades" action ToggleVariable("player_sageMode", 2) style "soundtest_button"', 4)
                elif widget.id == "volumenMusica":
                    writeLine(f, 'style "framesound"', 4)
                    writeLine(f, 'style_group "prefs"', 4)
                    writeLine(f, 'has vbox', 4)
                    name = "Volumen de la Musica"
                    writeLine(f, 'label "' + name + '"', 4)
                    writeLine(f, 'bar value Preference("music volume")', 4)
                elif widget.id == "volumenSonido":
                    writeLine(f, 'style "framesound"', 4)
                    writeLine(f, 'style_group "prefs"', 4)
                    writeLine(f, 'has vbox', 4)
                    name = "Volumen del Sonido"
                    writeLine(f, 'label "' + name + '"', 4)
                    writeLine(f, 'bar value Preference("sound volume")', 4)
                    writeLine(f, 'textbutton "Comprobar" action Play("sound", "sounds/sound_test.ogg") style "soundtest_button"', 4)
                elif widget.id == "volumenVoces":
                    writeLine(f, 'style "framesound"', 4)
                    writeLine(f, 'style_group "prefs"', 4)
                    writeLine(f, 'has vbox', 4)
                    name = "Volumen de las Voces"
                    writeLine(f, 'label "' + name + '"', 4)
                    writeLine(f, 'bar value Preference("voice volume")', 4)
                    writeLine(f, 'textbutton "Comprobar" action Play("voice", "sounds/voice_test.ogg") style "soundtest_button"', 4)

