
init python:
    
    import pygame
    
    #############################################################
    # CHiddenObjects
    # 
    #############################################################
    class CHiddenObjectScene(renpy.Displayable):

        #############################################################
        def __init__(self, **kwargs):

            renpy.Displayable.__init__(self, **kwargs)
                       
            # listen to events?
            self.sensitive = True
            
            # objects
            self.objects = [ ]
            
            # prevent click fraud
            #ui.timer(5, self.timerCheck, True)
            pygame.time.set_timer(pygame.USEREVENT + 1, 4000)
            self.clickFraudCounter = 0
            self.clickFraudCounterMax = 1
            self.fraudImage = None
            self.fraudImageX = 20
            self.fraudImageY = 20
            self.showFraudAlert = False
            
            self.objFrame = None
            
            self.finish = False
            
            # Colocar la posicion de cada Factory de acuerdo con el objeto asociado
            self.showParticles = []
            self.particlesXY = {}
            self.particles = {}
             
        #############################################################
        def show(self, layer='master'):
            pass            
            
        #############################################################
        def hide(self, layer='master'):
            pass
               
        #############################################################
        def append(self, piece, particleX, particleY, particleRadius):
            self.objects.append(piece)
            self.showParticles.append(False)
            piece.HOindex = len(self.showParticles) - 1
            self.particles[piece.HOindex] = RingParticles('images/misc/cursor_general.png', particleX, particleY, particleRadius)
            #~ self.particles[piece.HOindex] = RingParticles('images/misc/cursor_general.png', particleX, particleY, 100)
            self.particlesXY[piece.HOindex] = (particleX, particleY)
            #~ self.particles[piece.HOindex] = RingParticles('images/misc/cursor_general.png', 310, -20, 100)
            #~ self.particles[piece.HOindex] = Particles(MVExplodeFactory('images/misc/star-animation0000.png', particleX, particleY, explodeTime = 0.0, numParticles=10))            
            #self.particles[piece.HOindex] = Particles(MVExplodeFactory('images/star-animation0000.png', 150 / 800.0, 150 / 600.0))            
       
        #############################################################        
        def addFraudImage(self, fraudImage):
            self.fraudImage = renpy.easy.displayable_or_none(fraudImage)

        #############################################################        
        def addObjectFrame(self, objFrame):
            self.objFrame = renpy.easy.displayable_or_none(objFrame)

        #############################################################        
        def checkSolution(self):
            for object in self.objects:
                if object.active == True:
                    return 0
            return 1
            #~ return len(self.objects) == 0

        #############################################################
        def render(self, width, height, st, at):

            # get render object
            rv = renpy.Render(width, height)

            if self.objFrame != None:
                surf = renpy.render(self.objFrame, width, height, st, at)
                rv.blit(surf, (0, 0))

            for object in self.objects:
                    if object.alpha > 0.0:
                        object.render_to(rv, width, height, st, at)

            for i in xrange(len(self.showParticles)):
                    if self.showParticles[i] == True:
                        (px, py) = self.particlesXY[i]
                        #~ raise Exception(str(px))
                        surf = renpy.render(self.particles[i], width, height, st, at)
                        rv.blit(surf, (px, py))

            if self.showFraudAlert and self.fraudImage != None:
                surf = renpy.render(self.fraudImage, width, height, st, at)
                rv.blit(surf, (self.fraudImageX, self.fraudImageY))

            return rv
        
        #############################################################
        def event(self, ev, x, y, st):           
            
            if ev.type == pygame.USEREVENT + 1:
                if self.finish == True:
                    return 1                
                self.clickFraudCounter = 0
            elif ev.type == pygame.USEREVENT + 2:
                pygame.time.set_timer(pygame.USEREVENT + 2, 0)
                self.showFraudAlert = False
                self.sensitive = True
                renpy.redraw(self,0)
                return  
            #~ elif ev.type == pygame.USEREVENT + 3:
                #~ return 1
            #~ elif ev.type == pygame.USEREVENT + 4:                
            elif ev.type == pygame.USEREVENT + 3:
                changed = False
                #~ raise Exception('w')
                for object in self.objects:
                    if object.active == False and object.alpha >= 0.0:
                        #~ raise Exception('w')
                        changed = True
                        object.updateEffect()
                        if object.alpha <= 0.5:
                            self.showParticles[object.HOindex] = False
                if changed:
                    renpy.redraw(self,0)
                else:
                    pygame.time.set_timer(pygame.USEREVENT + 3, 0)
                #~ return
                

            if not self.sensitive:
                return
        
            if ev.type == pygame.MOUSEBUTTONDOWN and ev.button == 1 and self.showFraudAlert == False:
                
                # debug return
                #return 1
                
                for object in list(reversed(self.objects)):
                    if object.active == True and inRect(x, y, object.rect):
                        self.showParticles[object.HOindex] = True
                        object.active = False
                        #~ object.updateEffect()
                        renpy.music.play('sounds/sonido_logro.ogg', channel='sound', fadeout=0)
                        renpy.redraw(self,0)
                        solved = self.checkSolution()
                        pygame.time.set_timer(pygame.USEREVENT + 3, 100)
                        self.clickFraudCounter = 0
                        if solved == 1:
                            self.sensitive = False
                            self.finish = True
                            pygame.time.set_timer(pygame.USEREVENT + 1, 1500)
                            #~ return 1
                        return
                
                self.clickFraudCounter = self.clickFraudCounter + 1
                if self.clickFraudCounter >= self.clickFraudCounterMax:
                    self.sensitive = False
                    self.showFraudAlert = True
                    pygame.time.set_timer(pygame.USEREVENT + 2, 3000)
                    renpy.redraw(self,0)
            
    #############################################################
    #############################################################
    #############################################################
    
    class CHiddenObject(object):

        def __init__(self, hiddenObjectScene, glyph, wordGlyph, x=100, y=100, wx=100, wy=100):

            # The puzzle this piece belongs to.
            self.hiddenObjectScene = hiddenObjectScene
            
            # active
            self.active = True

            # Object image
            self.glyph = renpy.easy.displayable(glyph)
            #self.wordGlyph = renpy.easy.displayable(wordGlyph)
            self.wordGlyph = wordGlyph
            
            # Object ID in the HO scene
            self.HOindex = 0
                        
            # Location
            self.x = x
            self.y = y
            self.wx = wx
            self.wy = wy            
            self.rect = (0, 0, 0, 0)
            
            # visual attributes
            self.alpha = 1.0
            self.scale = 1.0
        
        def updateEffect(self):
            self.alpha -= 0.05
            #~ self.scale += 0.05            
            #~ raise Exception(str(self.alpha))
                        
        def render_to(self, rv, width, height, st, at):

            objdis = im.FactorScale(im.Alpha(self.glyph, self.alpha), self.scale)
            surf = renpy.render(objdis, width, height, st, at)
            #~ surf = renpy.render(im.Alpha(self.glyph, 0.5), width, height, st, at)
            w, h = surf.get_size()
            self.rect = (self.x, self.y, w, h)
            rv.blit(surf, (self.x, self.y))
            
            if self.alpha > 0.9:
                surf = renpy.render(self.wordGlyph, width, height, st, at)
                rv.blit(surf, (self.wx, self.wy))

    #############################################################
    #############################################################
    #############################################################
