
init python:
    
    import pygame
    import random
    
    def inRect(x, y, r):
        if r is None:
            return 0

        x1, y1, w1, h1 = r

        if (x >= x1) and (x <= x1 + w1) and (y >= y1) and (y <= y1 + h1):
            return 1
        
        return 0

    #############################################################
    # CPuzzle
    # Block (jigsaw) puzzles
    #############################################################
    class CPuzzle(renpy.Displayable):

        #############################################################
        def __init__(self, x=100, y=100, **kwargs):

            renpy.Displayable.__init__(self, **kwargs)
                       
            # listen to events?
            self.sensitive = True

            # puzzle pieces
            self.pieces = [ ]
            
            # puzzle piece currently being dragged
            self.draggingPiece = None
            self.draggingOffset = (0, 0)    # position of mouse relative to dragged piece
            
            # puzzle image
            self.imagePath = ''
            self.imageSize = None

            # puzzle area (board)
            self.areax = x
            self.areay = y

            # cells (the board has a dimension of areaRows x areaCols)
            self.areaColWidth = 0
            self.areaRowHeight = 0
            self.areaCols = 0
            self.areaRows = 0
            self.cellGap = 0                # gap (pixels) between adjacent cells
            self.defaultPieceGap = 0        # gap (pixels) between pieces after shuffling
            self.pieceDeckX = 0             # x of piece deck
            self.pieceDeckY = 0             # y of piece deck
            self.scalingImage = False       # scale the original image?
            self.scaleFactor = 0.0          # a scale factor (real number)
            self.imageEmptySlot = None      # image for empty slot (optional)
            self.alreadySolved = False      # set to True the first time afterSolve is invoked

            # matrix pointing to pieces in cells
            self.board = {}
                                              
        #############################################################
        def show(self, layer='master'):
            pass            
            
        #############################################################
        def hide(self, layer='master'):
            pass

        #############################################################
        def setActive(self, active):
            self.sensitive = active

        #############################################################
        def getImageSize(self, imagePath):
            
            if not renpy.exists(imagePath):
                return

            surface = pygame.image.load( renpy.loader.transfn(imagePath) )

            if surface is None:
                return 0, 0

            return surface.get_size()

        #############################################################
        def setImage(self, imagePath):

            if not renpy.exists(imagePath):
                return

            self.imagePath = imagePath
            self.imageSize = self.getImageSize(imagePath)

        #############################################################
        def setImageEmptySlot(self, imagePath):

            if not renpy.exists(imagePath):
                return

            self.imageEmptySlot = im.Scale( renpy.easy.displayable_or_none(imagePath), self.areaColWidth, self.areaRowHeight )

        #############################################################
        def scaleImage(self, factor):
            self.scalingImage = True
            self.scaleFactor = factor

        #############################################################
        def setupBoard(self, rows=5, cols=5):

            if rows < 1 or cols < 1:
                return

            if self.imageSize[0] == 0 or self.imageSize[1] == 0:
                return

            self.areaRows = rows
            self.areaCols = cols
            self.areaColWidth = self.imageSize[0] / self.areaCols
            self.areaRowHeight = self.imageSize[1] / self.areaRows

            self.board = {}
            self.pieces = []
            self.draggingPiece = None
            for i in xrange(self.areaRows):
                for j in xrange(self.areaCols):
                    self.board[i,j] = None
                    self.createPiece(self.imagePath, i, j)

            if self.scalingImage:
                self.imageSize = ( int(self.imageSize[0] * self.scaleFactor), int(self.imageSize[1] * self.scaleFactor) )
                self.areaColWidth = self.imageSize[0] / self.areaCols
                self.areaRowHeight = self.imageSize[1] / self.areaRows

        #############################################################
        def createPiece(self, imagePath, row, col):
            x = col * self.areaColWidth
            y = row * self.areaRowHeight

            pieceDisplayable = im.Crop(imagePath, x, y, self.areaColWidth, self.areaRowHeight)

            pieceWidth = self.areaColWidth
            pieceHeight = self.areaRowHeight

            if self.scalingImage:
                pieceWidth = int(self.scaleFactor * self.areaColWidth)
                pieceHeight = int(self.scaleFactor * self.areaRowHeight)
                pieceDisplayable = im.Scale( pieceDisplayable, pieceWidth, pieceHeight )

            piece = CPiece(pieceDisplayable, row, col, pieceWidth, pieceHeight)
            self.pieces.append(piece)
            return 

        #############################################################
        def setupPieces(self, deckX = 0, deckY = 0, cols=1):
            self.pieceDeckX = deckX
            self.pieceDeckY = deckY
            self.shufflePieces(cols)

        #############################################################
        def shufflePieces(self, cols=1):
            if cols < 1:
                return
            pieceCount = self.areaRows * self.areaCols
            if pieceCount < 1:
                return
            self.draggingPiece = None
            random.shuffle(self.pieces)
            
            for i in range(pieceCount):
                self.pieces[i].setPosition( (i % cols) * self.areaColWidth + self.defaultPieceGap + self.pieceDeckX, (i / cols) * self.areaRowHeight + self.defaultPieceGap + self.pieceDeckY)

            renpy.redraw(self, 0)

        #############################################################
        def getAreaCell(self, x, y):
            
            # compute cell (row,col) for point (x,y). A (-1-1) value implies an out-of-board point.
            
            if x < self.areax or y < self.areay:
                return (-1,-1)
            
            ox = x - self.areax
            oy = y - self.areay
            
            #if areaColWidth == 0 or areaRowHeight == 0:
            #    return

            col = ox / self.areaColWidth
            row = oy / self.areaRowHeight
            
            if row >= self.areaRows:
                return (-1,-1)
            
            if col >= self.areaCols:
                return (-1,-1)
            
            return (row, col)
        
        #############################################################
        def append(self, piece):
            self.pieces.append(piece)
   
        #############################################################
        def appendToCell(self, piece, row, col):
            self.pieces.append(piece)
            self.board[row,col] = piece
            piece.x = self.areax + col * (self.areaColWidth + self.cellGap)
            piece.y = self.areay + row * (self.areaRowHeight + self.cellGap)
            piece.row = row
            piece.col = col

        #############################################################            
        def boardCompleted(self):
            for i in range(self.areaRows):
                for j in range(self.areaCols):
                    if self.board[i,j] == None:
                        return False
                    if self.board[i,j].row != self.board[i,j].rowSolution or self.board[i,j].col != self.board[i,j].colSolution:
                        return False
            return True
        
        #############################################################        
        def solved(self):
            result = self.boardCompleted()
            return result        

        #############################################################
        def afterSolve(self):
            self.alreadySolved = True
            self.setActive(False)
            return

        #############################################################
        def render(self, width, height, st, at):

            # get render object
            rv = renpy.Render(width, height)
                    
            if self.imageEmptySlot:
                for i in range(self.areaRows):
                    for j in range(self.areaCols):
                        if self.board[i,j] == None:
                            surf = renpy.render(self.imageEmptySlot, width, height, st, at)
                            rv.blit(surf, ( j * self.areaColWidth + self.areax, i * self.areaRowHeight + self.areay ) )

            # La pieza que está actualmente seleccionada (si alguna lo está) se dibuja de última para
            # garantizar su sobreposición
            for c in self.pieces:
                if c != self.draggingPiece:
                    c.render_to(rv, width, height, st, at)
                        
            if self.draggingPiece != None:
                self.draggingPiece.render_to(rv, width, height, st, at)

            return rv
        
        #############################################################
        def event(self, ev, x, y, st):           
            
            if self.alreadySolved and ev.type == pygame.KEYDOWN:
                return 1

            if not self.sensitive:
                return

            if ev.type == pygame.MOUSEBUTTONDOWN and ev.button == 1:
                
                if self.draggingPiece == None:
                    # La inspección de la pieza seleccionada debe hacerse en orden inverso al dibujado, porque las
                    # piezas en posiciones superiores (on-top) tienen prioridad para detección de eventos click.
                    # Es decir, la pieza que se dibuja de última (está al final de la lista de piezas) tiene mayor
                    # prioridad para detección de click.
                    #for c in self.pieces:
                    for c in list(reversed(self.pieces)):
                        if inRect(x, y, c.rect):
                            self.pieces.remove(c)
                            self.pieces.append(c)
                            self.draggingPiece = c
                            self.draggingOffset = (x - self.draggingPiece.x, y - self.draggingPiece.y)
                            if self.draggingPiece.row > -1:
                                self.board[self.draggingPiece.row, self.draggingPiece.col] = None
                                self.draggingPiece.row = self.draggingPiece.col = -1
                            renpy.redraw(self,0)
                            break
                else:                    
                    row, col = self.getAreaCell(x, y)
                    if row != -1 and col != -1:
                        self.draggingPiece.x = self.areax + col * (self.areaColWidth + self.cellGap)
                        self.draggingPiece.y = self.areay + row * (self.areaRowHeight + self.cellGap)
                        self.draggingPiece.row = row
                        self.draggingPiece.col = col
                        #renpy.music.play('sounds/sonido_logro.ogg', channel='sound', fadeout=0)
                        if self.board[row,col] != None:
                            tmp = self.board[row,col]
                            self.board[row,col] = self.draggingPiece
                            self.draggingPiece = tmp
                            self.draggingPiece.row = -1
                            self.draggingPiece.col = -1           
                            self.draggingPiece.x = x - self.draggingOffset[0]
                            self.draggingPiece.y = y - self.draggingOffset[1]
                        else:
                            self.board[row,col] = self.draggingPiece
                            self.draggingPiece = None
                            #check if puzzle solved
                            if self.solved():
                                renpy.redraw(self, 0)
                                return self.afterSolve()
                    else:
                        self.draggingPiece = None
                    renpy.redraw(self,0)

            if (ev.type == pygame.MOUSEMOTION) and (self.draggingPiece != None):
                self.draggingPiece.x = x - self.draggingOffset[0]
                self.draggingPiece.y = y - self.draggingOffset[1]
                renpy.redraw(self,0)

    #############################################################
    #############################################################
    #############################################################
    
    class CPiece(object):

        def __init__(self, displayable, rowSolution, colSolution, width=0, height=0, x=100, y=100):
            # The face of this piece
            self.displayable = displayable
            
            # Location
            self.x = x
            self.y = y
            self.width = width
            self.height = height
            self.rect = (x, y, width, height)
            
            # Index in board
            self.row = -1
            self.col = -1
            
            # Index for solution (the index this piece should occupy in the solved puzzle)
            self.rowSolution = rowSolution
            self.colSolution = colSolution

        def setPosition(self, x, y):
            self.x = x
            self.y = y
            self.rect = (x, y, self.width, self.height)

        def render_to(self, rv, width, height, st, at):
            surf = renpy.render(self.displayable, width, height, st, at)
            #w, h = surf.get_size()
            self.rect = (self.x, self.y, self.width, self.height)
            rv.blit(surf, (self.x, self.y))

    #############################################################
    #############################################################
    #############################################################
